/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.ajde;

import org.eclipse.core.runtime.QualifiedName;
import java.util.Locale;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ajdt.ui.AspectJPlugin;

public class BuildOptionsAdapter
	implements org.aspectj.ajde.BuildOptionsAdapter {

	public final static QualifiedName USE_JAVAC =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.useJavac");
	public final static QualifiedName PREPROCESS =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.preProcess");
	public final static QualifiedName SOURCE14 =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.source14");
	public final static QualifiedName PORTING_MODE =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.portingMode");
	public final static QualifiedName WORKING_DIR =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.workingDirectory");
		// Temporarily removing this option, will return nothing to the caller.
	public final static QualifiedName CHAR_ENC =
	
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.characterEncoding");
	public final static QualifiedName NON_STANDARD_OPTS =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.nonStandardOptions");
	public final static QualifiedName COMPILATION_STRICTNESS =
		new QualifiedName(
			AspectJPlugin.PLUGIN_ID,
			"BuildOptions.compilationStrictness");

	// Enumeration of values for COMPILATION_STRICTNESS key
	public final static String COMPILATION_STRICTNESS_NORMAL = "NORMAL";
	public final static String COMPILATION_STRICTNESS_LENIENT = "LENIENT";
	public final static String COMPILATION_STRICTNESS_STRICT = "STRICT";

	// Default values for those options that are easy to default...
	public final static boolean USE_JAVAC_DEFAULT = false;
	public final static boolean PREPROCESS_DEFAULT = false;
	public final static boolean SOURCE14_DEFAULT = false;
	public final static boolean PORTING_MODE_DEFAULT = false;
	public final static String COMPILATION_STRICTNESS_DEFAULT =
		COMPILATION_STRICTNESS_NORMAL;
	public final static String NON_STANDARD_OPTS_DEFAULT = "-Xlint";
	public final static String CHAR_ENC_DEFAULT =
		System.getProperty("file.encoding");
	//	java.util.Locale.getDefault().getDisplayName();

	// Return formatted version of the current build options set
	public String toString() {
		StringBuffer formattedOptions = new StringBuffer();
		formattedOptions.append("Current Compiler options set:");
		formattedOptions.append(
			"[Use javac for compilation=" + getUseJavacMode() + "]");
		formattedOptions.append("[Just preprocess=" + getPreprocessMode() + "]");
		formattedOptions.append(
			"[Allow 1.4 assertions=" + getSourceOnePointFourMode() + "]");
		formattedOptions.append("[Porting mode=" + getPortingMode() + "]");
		formattedOptions.append("[WorkingDirectory='" + getWorkingOutputPath() + "']");
		formattedOptions.append(
			"[NonStandard options='" + getNonStandardOptions() + "']");
		if (getLenientSpecMode())
			formattedOptions.append("[Compilation Strictness=LENIENT]");
		else if (this.getStrictSpecMode())
			formattedOptions.append("[Compilation Strictness=STRICT]");
		else
			formattedOptions.append("[Compilation Strictness=NORMAL]");

		return formattedOptions.toString();
	}

	private boolean retrieveSettingBoolean(QualifiedName key) {

		IProject thisProject = AspectJPlugin.getDefault().getCurrentProject();

		try {
			String value = thisProject.getPersistentProperty(key);

			boolean valueB = new Boolean(value).booleanValue();
			return valueB;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("buildOptionsAdapter.exceptionDuringRetrieve"),
				ce);
		}
		return false;
	}

	private String retrieveSettingString(QualifiedName key) {

		IProject thisProject = AspectJPlugin.getDefault().getCurrentProject();
		try {
			String value = thisProject.getPersistentProperty(key);
			return value;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("buildOptionsAdapter.exceptionDuringRetrieve"),
				ce);
		}
		return "";
	}

	/**
	 * @see BuildOptionsAdapter#getCharacterEncoding()
	 */
	public String getCharacterEncoding() {
		ensurePropertiesInitialized();
		
		return "";
		/*
		 * Currently commented out since it has no meaning running in Eclipse 
		String encoding = retrieveSettingString(CHAR_ENC);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getCharacterEncoding called, returning :" + encoding);
		}
		return encoding;
		*/
	}

	/**
	 * @see BuildOptionsAdapter#getLenientSpecMode()
	 */
	public boolean getLenientSpecMode() {
		ensurePropertiesInitialized();

		String compilationStrictness = retrieveSettingString(COMPILATION_STRICTNESS);
		boolean lenientMode =
			compilationStrictness.equals(COMPILATION_STRICTNESS_LENIENT);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getLenientSpecMode called, returning :"
					+ new Boolean(lenientMode));
		}

		return lenientMode;
	}

	/**
	 * @see BuildOptionsAdapter#getNonStandardOptions()
	 */
	public String getNonStandardOptions() {
		ensurePropertiesInitialized();

		String nonStandardOptions = retrieveSettingString(NON_STANDARD_OPTS);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getNonStandardOptions called, returning :"
					+ nonStandardOptions);
		}
		return nonStandardOptions;
	}

	/**
	 * @see BuildOptionsAdapter#getPortingMode()
	 */
	public boolean getPortingMode() {
		ensurePropertiesInitialized();

		boolean portingMode = retrieveSettingBoolean(PORTING_MODE);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getPortingMode called, returning :"
					+ new Boolean(portingMode));
		}
		return portingMode;
	}

	/**
	 * @see BuildOptionsAdapter#getPreprocessMode()
	 */
	public boolean getPreprocessMode() {
		ensurePropertiesInitialized();

		boolean preprocessMode = retrieveSettingBoolean(PREPROCESS);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getPreprocessMode called, returning :"
					+ new Boolean(preprocessMode));
		}
		return preprocessMode;

	}

	/**
	 * @see BuildOptionsAdapter#getSourceOnePointFourMode()
	 */
	public boolean getSourceOnePointFourMode() {
		ensurePropertiesInitialized();

		boolean source14 = retrieveSettingBoolean(SOURCE14);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getSourceOnePointFourMode called, returning :"
					+ new Boolean(source14));
		}
		return source14;

	}

	/**
	 * @see BuildOptionsAdapter#getStrictSpecMode()
	 */
	public boolean getStrictSpecMode() {
		ensurePropertiesInitialized();

		String compilationStrictness = retrieveSettingString(COMPILATION_STRICTNESS);
		boolean strictMode =
			compilationStrictness.equals(COMPILATION_STRICTNESS_STRICT);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getStrictSpecMode called, returning :"
					+ new Boolean(strictMode));
		}

		return strictMode;

	}

	/**
	 * @see BuildOptionsAdapter#getUseJavacMode()
	 */
	public boolean getUseJavacMode() {
		ensurePropertiesInitialized();

		boolean useJavacMode = retrieveSettingBoolean(USE_JAVAC);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getUseJavacMode called, returning :"
					+ new Boolean(useJavacMode));
		}

		return useJavacMode;
	}

	/**
	 * @see BuildOptionsAdapter#getWorkingOutputPath()
	 */
	public String getWorkingOutputPath() {
		ensurePropertiesInitialized();

		String workingDir = retrieveSettingString(WORKING_DIR);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getWorkingOutputPath called, returning :" + workingDir);
		}

		return workingDir;
	}

	/**
	* Static form of preserveSetting that takes a project name, a key and a boolean,
	* it then preserves the boolean value for the named key on the supplied project resource.
	* Actually converts the boolean to a string and delegates to the 
	* other implementation of preserveSetting() that takes a string as the
	* value to store.
	*/
	private static void preserveSetting(
		IProject project,
		QualifiedName key,
		boolean flag)
		throws CoreException {
		preserveSetting(project, key, new Boolean(flag).toString());
	}

	/**
	 * Static form of preserveSetting that takes a project name, a key and a value,
	 * it then preserves the key/value pair as a persistent property against the 
	 * supplied project resource.
	 */
	private static void preserveSetting(
		IProject project,
		QualifiedName key,
		String value)
		throws CoreException {
		project.setPersistentProperty(key, value);
	}

	/** 
	 * Ensure the properties are initialized before they are used.
	 */
	private void ensurePropertiesInitialized() {
		ensurePropertiesInitialized(AspectJPlugin.getDefault().getCurrentProject());
	}

	public static void ensurePropertiesInitialized(IProject project) {
		try {
			if (project.getPersistentProperty(BuildOptionsAdapter.USE_JAVAC) == null)
				preserveSetting(
					project,
					BuildOptionsAdapter.USE_JAVAC,
					BuildOptionsAdapter.USE_JAVAC_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.PREPROCESS) == null)
				preserveSetting(
					project,
					BuildOptionsAdapter.PREPROCESS,
					BuildOptionsAdapter.PREPROCESS_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.SOURCE14) == null)
				preserveSetting(
					project,
					BuildOptionsAdapter.SOURCE14,
					BuildOptionsAdapter.SOURCE14_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.WORKING_DIR) == null)
				// ASCFIXME - Is this the right default?
				preserveSetting(
					project,
					BuildOptionsAdapter.WORKING_DIR,
					project.getLocation().toOSString());
			if (project.getPersistentProperty(BuildOptionsAdapter.CHAR_ENC) == null)
				// Is this right? It comes out as "English" when perhaps it ought to be "ISO8859-1"
				preserveSetting(
					project,
					BuildOptionsAdapter.CHAR_ENC,
					BuildOptionsAdapter.CHAR_ENC_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.COMPILATION_STRICTNESS)
				== null)
				preserveSetting(
					project,
					BuildOptionsAdapter.COMPILATION_STRICTNESS,
					BuildOptionsAdapter.COMPILATION_STRICTNESS_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.PORTING_MODE) == null)
				preserveSetting(
					project,
					BuildOptionsAdapter.PORTING_MODE,
					BuildOptionsAdapter.PORTING_MODE_DEFAULT);
			if (project.getPersistentProperty(BuildOptionsAdapter.NON_STANDARD_OPTS)
				== null)
				preserveSetting(
					project,
					BuildOptionsAdapter.NON_STANDARD_OPTS,
					BuildOptionsAdapter.NON_STANDARD_OPTS_DEFAULT);

		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString(
					"buildOptionsAdapter.exceptionInitializingProperties"),
				ce);
		}
	}

}