/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
 
   AMC 08/12/2002  Changed getAspectjrtClasspath to get version info 
                   from Ajde instead of being hard-coded.

**********************************************************************/
package org.eclipse.ajdt.internal.ui.ajde;

import java.util.*;
import java.util.List;
import java.io.File;
import org.aspectj.ajde.ProjectPropertiesAdapter;

import org.eclipse.jface.viewers.*;
import org.eclipse.jface.window.*;
import org.eclipse.jface.action.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ajdt.internal.ui.*;
import org.eclipse.ajdt.internal.ui.AspectJPreferences;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;
import org.eclipse.jface.preference.*;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.preferences.CompilerPreferencePage;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.aspectj.ajde.*;

/**
 * ProjectProperties is used to pass all the user, project and plugin settings to 
 * AJ Tools.
 */
public class ProjectProperties implements ProjectPropertiesAdapter {

	/**
	 * A map from java.io.File to the corresponding IResource in the project.
	 */
	private Hashtable fileToResourceHt = null;

	/**
	 * Computed classpath to aspectjrt.jar
	 */
	private String aspectjrtPath = null;


	/*
	 * @see ProjectPropertiesAdapter#getAjcWorkingDir()
	 */
	public String getAjcWorkingDir() {
		return AspectJPlugin.getDefault( ).getWorkspace().getRoot().getLocation().toOSString();
	}

	/**
	 * Called to determine where the resultant class files should
	 * go when AJC is compiling code.  We grab the location from
	 * the current project and convert it to a string.
	 */
	public String getOutputPath() {
		try {
			IProject currProject = AspectJPlugin.getDefault().getCurrentProject();
			
			// Bug21998: Use the project location rather than the workspace location.
			String workspaceLoc = currProject.getLocation().removeLastSegments(1).toOSString();
			
			IJavaProject jProject = JavaCore.create(currProject);
			String relativePath = jProject.getOutputLocation().toOSString();
			return workspaceLoc + relativePath;
		} catch (JavaModelException jme) {
			Ajde.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString( "noOutputDir" ),
				jme);
		}
		return null;
	}

	/**
	 * The name of the current project
	 */
	public String getProjectName() {
		return AspectJPlugin.getDefault().getCurrentProject().getName();
	}

	/*
	 * @see ProjectPropertiesAdapter#getClassToExecute()
	 */
	public String getClassToExecute() {
		System.err.println( "*** AJ Plugin: ProjectProperties.getClassToExecute NOT IMPLEMENTED ***" );
		return null;
	}
	
	
	/**
	 * The home directory of the current project
	 */
	public String getRootProjectDir() {
		IProject project = AspectJPlugin.getDefault().getCurrentProject();
		return project.getLocation().toOSString();
	}

	/**
	 * All the source files in the current project, as a List of
	 * java.io.Files.
	 */
	public List getProjectSourceFiles() {
		IProject activeProject = AspectJPlugin.getDefault().getCurrentProject();
		return getProjectSourceFiles( activeProject );
	}
	
	/**
	 * version to use when you know the project
	 */
	public List getProjectSourceFiles( IProject project) {

		List allProjectSourceFiles = new ArrayList();
		fileToResourceHt = new Hashtable();

		// bug 23045 - we don't want every file in the project, just those in
		// the active source directories	
		IJavaProject jProject = JavaCore.create(project);		
		try {				
			IClasspathEntry[] classpathEntries = jProject.getResolvedClasspath(false);
			for (int i = 0; i < classpathEntries.length; i++) {
				if (classpathEntries[i].getEntryKind() == IClasspathEntry.CPE_SOURCE ) {
					IClasspathEntry sourceEntry = classpathEntries[i];
					IPath sourcePath = sourceEntry.getPath();
					// remove the first segment because the findMember call following
					// always adds it back in under the covers (doh!) and we end up
					// with two first segments otherwise!
					sourcePath = sourcePath.removeFirstSegments( 1 ); 					
					IResource[] srcContainer = new IResource[] {project.findMember( sourcePath )};
					getAllFiles(srcContainer, allProjectSourceFiles, fileToResourceHt);
				}
			}
		} catch (JavaModelException jmEx) {
			String message = AspectJPlugin.getResourceString("jmCoreException");
			Status status =
				new Status(Status.ERROR, AspectJPlugin.PLUGIN_ID, Status.OK, message, jmEx);
			Shell shell = AspectJPlugin.getDefault().getActiveWorkbenchWindow().getShell();
			ErrorDialog.openError(
				shell,
				AspectJPlugin.getResourceString("ajErrorDialogTitle"),
				message,
				status);
		}
			
		return allProjectSourceFiles;
	}

	/*
	 * @see ProjectPropertiesAdapter#getProjectSourcePath()
	 */
	public String getProjectSourcePath() {
		IProject p = AspectJPlugin.getDefault().getCurrentProject();
		// todo - how to tell if the root of the project source tree??
		// is it a getPersistentProperty( )??
		return p.getLocation().toOSString();
	}

	/**
	 * get the classpath to use for compiling the current project.
	 */
	public String getClasspath() {
		StringBuffer classpath = new StringBuffer();
		
		// Related to bug21998: Use the project location rather than the workspace location, this allows
		// for projects who keep their source/binaries in a location other than the workspace directories.
		//String prependPath = AspectJPlugin.getWorkspace().getRoot().getLocation().toOSString();
		IProject currProject = AspectJPlugin.getDefault().getCurrentProject();
		String prependPath = currProject.getLocation().removeLastSegments(1).toOSString();

		classpath.append(getAspectjrtClasspath());
		classpath.append(File.pathSeparator);
		IProject project = AspectJPlugin.getDefault().getCurrentProject();
		
		if (project != null) {
			IJavaProject jProject = JavaCore.create(project);		
			String projectLocalPrefix = File.separator + project.getName();
			try {				
				IClasspathEntry[] classpathEntries = jProject.getResolvedClasspath(false);
				for (int i = 0; i < classpathEntries.length; i++) {
					
					String cpstring = classpathEntries[i].getPath( ).toOSString();	
					// what to do next depends on the entry kind
					switch ( classpathEntries[i].getEntryKind( ) ) {					
					
						case IClasspathEntry.CPE_LIBRARY:
							// could be an external jar (fully qualified),
							// or a local jar (starts with our project name).
							// the latter needs qualifying for ajc						
							if ( cpstring != null && cpstring.startsWith( projectLocalPrefix ) ) {
								cpstring = prependPath + cpstring;								
							}
							break;
						
						case IClasspathEntry.CPE_PROJECT:
							// project path will be relative to the workspace,
							// needs qualification
							cpstring = prependPath + cpstring;
							break;
						
						case IClasspathEntry.CPE_SOURCE:
							// a source directory under my project, needs
							// qualification with the base location for the project.
							cpstring = prependPath + cpstring;
							break;
						
						case IClasspathEntry.CPE_VARIABLE:						
						case IClasspathEntry.CPE_CONTAINER:
							// these two should already be qualified through use of
							// getResolvedClasspath						
						default:
							// do nothing
					}
					classpath.append( cpstring );
					classpath.append( File.pathSeparator );
				}
			} catch (JavaModelException jmEx) {
				String message = AspectJPlugin.getResourceString("jmCoreException");
				Status status =
					new Status(Status.ERROR, AspectJPlugin.PLUGIN_ID, Status.OK, message, jmEx);
				Shell shell = AspectJPlugin.getDefault().getActiveWorkbenchWindow().getShell();
				ErrorDialog.openError(
					shell,
					AspectJPlugin.getResourceString("ajErrorDialogTitle"),
					message,
					status);
			}
		}
		if (AspectJPlugin.isDebugging) {
			// It's *always* a classpath problem ;-). Better print it out.
			System.out.println("Using classpath: " + classpath);
		}
		return classpath.toString();
	}

	/*
	 * @see ProjectPropertiesAdapter#getBootClasspath()
	 */
	public String getBootClasspath() {
		return null;
	}


	/*
	 * @see ProjectPropertiesAdapter#getExecutionArgs()
	 */
	public String getExecutionArgs() {
		String options = AspectJPreferences.getCompilerOptions();
		return options;
	}


	/*
	 * @see ProjectPropertiesAdapter#getVmArgs()
	 */
	public String getVmArgs() {
		return null;
	}

	// ----------- end of ProjectPropertiesAdapter interface methods ------------


	/**
	 * Called from builder before doing a build in order to 
	 * clear all problem markers
	 */
	public void clearMarkers() {
		IProject currProject = AspectJPlugin.getDefault().getCurrentProject();
		try {
			currProject.deleteMarkers( IMarker.PROBLEM, true, IResource.DEPTH_INFINITE );
		} catch (Exception ex ) {}
	}

	/**
	 * Return the IResource within the project that maps to the
	 * given File
	 */
	public IResource findResource( String fullPath ) {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot( );
		IPath path = new Path( fullPath ); 
		return root.getFileForLocation( path );
	}

	/**
	 * Get the aspectjrt.jar classpath entry. This is in 
	 * plugins/org.aspectj.ajde_1.0.5/aspectjrt.jar
	 */
	private String getAspectjrtClasspath() {
		if (aspectjrtPath == null) {
			StringBuffer cpath = new StringBuffer();
			IPath rootPath =
				AspectJPlugin.getDefault().getWorkspace().getRoot().getLocation();
			IPath installPath = rootPath.removeLastSegments(1);
			// chop of the "workspace" directory
			cpath.append(installPath.toOSString());
			cpath.append(File.separator);
			cpath.append("plugins");
			cpath.append(File.separator);
			
			//AMC - change from hard-coded version to use of getVersion( )
			//      method in Ajde. This gives us some flexibility across
			//      AJDE versions.
			//cpath.append("org.aspectj.ajde_1.0.5");
			cpath.append("org.aspectj.ajde_");
			cpath.append( Ajde.getDefault().getVersion() );
			
			cpath.append(File.separator);
			cpath.append("aspectjrt.jar");
			aspectjrtPath = cpath.toString();
		}
		return aspectjrtPath;
	}


	/**
	 * Find all the ".java" and ".aj" files in the project.
	 */
	private void getAllFiles(
		IResource[] resource_list,
		List allProjectFiles,
		Hashtable fileToResourceHt) {
		try {
			for (int i = 0; i < resource_list.length; i++) {
				IResource ir = resource_list[i];
				if (ir.getName().endsWith(".java") || ir.getName( ).endsWith(".aj") ) {
					allProjectFiles.add(new File(ir.getLocation().toOSString()));
					if (fileToResourceHt!=null)
					  fileToResourceHt.put(new File(ir.getLocation().toOSString()), ir);
				}
				if (ir instanceof IContainer) {
					getAllFiles(((IContainer) ir).members(), allProjectFiles, fileToResourceHt);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
	
	/** New interface methods follow for build configuration management */
	
	/**
	 * Return a List containing strings, each string is the full path to 
	 * a build configuration file.
	 */
	public List getBuildConfigFiles() {
		List lstFiles_Strings = new ArrayList();
        List lstFiles_IResources = AspectJPlugin.getDefault().getListOfConfigFilesForCurrentProject();
        // Convert the IResource list to a list of strings for the full paths
        Iterator iter = lstFiles_IResources.iterator();
        IResource ir;
        while (iter.hasNext()) {
        	
        	ir= (IResource)iter.next();
        	//System.err.println("AC_temp_debug: ProjectProperties.getBuildConfigFiles(): Path to a .lst is "+ir.getFullPath().toOSString());
        	lstFiles_Strings.add(ir.getFullPath().toOSString());
        }
		return lstFiles_Strings;
	}
	
	public String getDefaultBuildConfigFile() {
			
		String defaultLstFile =  (AspectJPlugin.getDefault().getCurrentProject().getLocation().toOSString()
					+ File.separator
					+ org.eclipse.ajdt.internal.builder.Builder.DEFAULT_CONFIG_FILE);
		//System.err.println("AC_temp_debug: ProjectProperties.getDefaultBuildConfigFile(): Returning "+defaultLstFile);
        			
		return defaultLstFile;
		//return defaultLstFile + "TMP FIX DUE TO BUG IN AJDE";
	}
	
	public String getLastActiveBuildConfigFile() {
		
		String currentLstFile = AspectJPlugin.getDefault().getBuildConfigurationFile(AspectJPlugin.getDefault().getCurrentProject());
		//System.err.println("AC_temp_debug: ProjectProperties.getLastActiveBuildConfigFile(): Returning "+currentLstFile);
		return currentLstFile;
	}
}