/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer - <description of contribution>
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.wizards;

import org.eclipse.jdt.internal.ui.wizards.NewElementWizard;
//asc import org.eclipse.jdt.internal.ui.wizards.NewClassCreationWizardPage;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IFile;
import java.io.*;
import org.eclipse.core.resources.IWorkspace;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.JavaPluginImages;

import org.eclipse.ajdt.ui.AspectJPlugin;
import org.aspectj.ajde.ui.AbstractIcon;
import org.aspectj.asm.ProgramElementNode;
import org.eclipse.ajdt.internal.core.resources.AJDTIcon;
import org.eclipse.ajdt.internal.core.resources.AspectJImages;

/**
 * This wizard is used to create a new aspect. It bases from the 
 * *internal* JDT class creation wizard - ideally this would be 
 * exposed as a wizard we could legitimately extend.
 */
public class NewAspectWizard extends NewElementWizard {

	private NewAspectCreationWizardPage fPage;

	public NewAspectWizard() {
		super();
		AbstractIcon aspectIcon =
			AspectJImages.registry().getStructureIcon(ProgramElementNode.Kind.ADVICE);
		setDefaultPageImageDescriptor(((AJDTIcon) aspectIcon).getImageDescriptor());
		setDialogSettings(JavaPlugin.getDefault().getDialogSettings());
		setWindowTitle(
			AspectJPlugin.getResourceString("NewAspectCreationWizard.title"));
	}

	/*
	 * @see Wizard#createPages
	 */
	public void addPages() {
		super.addPages();
		IWorkspace workspace = JavaPlugin.getWorkspace();
		fPage = new NewAspectCreationWizardPage();
		addPage(fPage);
		fPage.init(getSelection());
	}

	/*
	 * Although this class is almost a duplicated of the wizard for creating new
	 * classes, the performFinish method *is* different.  Normally there are only
	 * two kinds of resource that can be created by the pages and underlying JDT
	 * infrastructure being reused in this wizard - either a class is being
	 * created or an interface.  It is the performFinish method that alters the
	 * class produced by the underlying wizard code such that it will say "aspect MyAspect"
	 * rather than "class MyAspect".
	 * 
	 * @see Wizard#performFinish
	 */
	public boolean performFinish() {
		if (finishPage(fPage.getRunnable())) {
			ICompilationUnit cu = fPage.getCreatedType().getCompilationUnit();
			if (cu.isWorkingCopy()) {
				cu = (ICompilationUnit) cu.getOriginalElement();
			}

			try {
				IResource resource = cu.getUnderlyingResource();

				// Here is where the aspect wizard differs from the JDT class wizard.
				// Once the resource is obtained, the contents are altered such that
				// it says "aspect <abcdef>" rather than "class <abcdef>"

				// First cast the resource to a file
				IFile ifile = ((IFile) resource);

				try {
					// Grab a copy of the file into a byte array
					InputStream is = ifile.getContents();
					byte[] inputdata = new byte[is.available()];
					is.read(inputdata);

					// Create a string representation of the current file contents
					String currentContents = new String(inputdata);
					String newcontents = "";
					
					// It is difficult to identify what to replace in the contents of the source generated
					// by the underlying NewTypeWizardPage code.  The simple phrase searched for here is
					// "class <TypeName>" - so a type called "Lemon" is being created, this snippet of
					// code looks for "class Lemon" and replaces the word "class" with the word
					// "aspect".
					// This is better than just looking for the first occurrence of "class" - but it could
					// still fail - if, for example, the javadoc header created within the source file
					// contains the "class Lemon" statement!  The phrase search could be made more sophisticated
					// by grabbing the whole line with the "class Lemon" phrase on it and ensuring
					// there is no "//" "/*" or "*" ahead of the "class " phrase found -
					// if any of those 'comment' markers are found, try the next occurrence of class

					String phraseToLocate = new String("class "+fPage.getTypeName());
					int i = currentContents.indexOf(phraseToLocate);
					if (i != -1) {
						newcontents =
							new String(
								currentContents.substring(0, i) + "aspect " + currentContents.substring(i + 6)); // 6 = length of "class "

					} else {
						// Bugger!!!!
						System.err.println("class keyword not found in (" + currentContents + ")");
					}

					// Build an appropriate input stream to pass to setContents()
					ByteArrayInputStream bais = null;
					bais = new ByteArrayInputStream(newcontents.getBytes());

					// Alter the contents
					ifile.setContents(bais, false, false, null);
				} catch (Exception e) {
					AspectJPlugin.getDefault().getErrorHandler().handleError(
						AspectJPlugin.getResourceString(
							"NewAspectCreationWizard.exceptionChangingFileContents")
							+ ifile.getFullPath().toOSString(),
						e);
				}
				selectAndReveal(resource);
				openResource(resource);
			} catch (JavaModelException e) {
				// Could skip reporting this, its not the end of the world.
				AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("NewAspectCreationWizard.exceptionDuringFinish"),e);
			}
			return true;
		}
		return false;
	}

}